﻿using System.Collections.Generic;
using System.IO;
using System.IO.Compression; 

namespace TestProject
{
    /// <summary>
    /// ZIP 파일 헬퍼
    /// </summary>
    public static class ZIPFileHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region ZIP 파일 생성하기 - CreateZIPFile(sourceDirectoryPath, targetFilePath, exceptionDirectoryPathList)

        /// <summary>
        /// ZIP 파일 생성하기
        /// </summary>
        /// <param name="sourceDirectoryPath">소스 디렉토리 경로</param>
        /// <param name="targetFilePath">타겟 파일 경로</param>
        /// <param name="exceptionDirectoryPathList">제외 디렉토리 경로</param>
        public static void CreateZIPFile(string sourceDirectoryPath, string targetFilePath, List<string> exceptionDirectoryPathList = null)
        {
            if(File.Exists(targetFilePath))
            {
                File.SetAttributes(targetFilePath, FileAttributes.Normal);

                File.Delete(targetFilePath);
            }

            using(FileStream fileStream = new FileStream(targetFilePath, FileMode.Create, FileAccess.ReadWrite))
            {
                using(ZipArchive zipArchive = new ZipArchive(fileStream, ZipArchiveMode.Create))
                {
                    foreach(string directoryPath in Directory.EnumerateDirectories(sourceDirectoryPath, "*", SearchOption.AllDirectories))
                    {
                        string relativePath = directoryPath.Substring(sourceDirectoryPath.Length + 1).Replace("\\", "/") + "/"; 

                        if(CanExcept(exceptionDirectoryPathList, relativePath))
                        {
                            continue;
                        } 

                        zipArchive.CreateEntry(relativePath);
                    } 

                    foreach(string filePath in Directory.EnumerateFiles(sourceDirectoryPath, "*.*", SearchOption.AllDirectories))
                    {
                        string relativePath = filePath.Substring(sourceDirectoryPath.Length + 1); 

                        try
                        {
                            zipArchive.CreateEntryFromFile(filePath, relativePath);
                        }
                        catch(PathTooLongException)
                        {
                        }
                    }
                }
            }
        }

        #endregion
        #region ZIP 파일 추출하기 - ExtractZIPFile(sourceFilePath, targetDirectoryPath)

        /// <summary>
        /// ZIP 파일 추출하기
        /// </summary>
        /// <param name="sourceFilePath">소스 파일 경로</param>
        /// <param name="targetDirectoryPath">타겟 디렉토리 경로</param>
        public static void ExtractZIPFile(string sourceFilePath, string targetDirectoryPath)
        {
            if(!Directory.Exists(targetDirectoryPath))
            {
                Directory.CreateDirectory(targetDirectoryPath);
            }

            using(ZipArchive zipArchive = ZipFile.OpenRead(sourceFilePath))
            {
                foreach(ZipArchiveEntry zipArchiveEntry in zipArchive.Entries)
                {
                    try
                    {
                        string folderPath = Path.GetDirectoryName(Path.Combine(targetDirectoryPath, zipArchiveEntry.FullName)); 

                        if(!Directory.Exists(folderPath))
                        {
                            Directory.CreateDirectory(folderPath);
                        } 

                        if(zipArchiveEntry.FullName.EndsWith("/"))
                        {
                            continue;
                        } 

                        zipArchiveEntry.ExtractToFile(Path.Combine(targetDirectoryPath, zipArchiveEntry.FullName));
                    }
                    catch(PathTooLongException)
                    {
                    }
                }
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 제외 여부 구하기 - CanExcept(exceptionDirectoryPathList, relativePath)

        /// <summary>
        /// 제외 여부 구하기
        /// </summary>
        /// <param name="exceptionDirectoryPathList">제외 디렉토리 경로 리스트</param>
        /// <param name="relativePath">상대 경로</param>
        /// <returns>제외 여부</returns>
        private static bool CanExcept(List<string> exceptionDirectoryPathList, string relativePath)
        {
            if(exceptionDirectoryPathList == null || exceptionDirectoryPathList.Count == 0)
            {
                return false;
            } 

            foreach(string value in exceptionDirectoryPathList)
            {
                if(relativePath.StartsWith(value))
                {
                    return true;
                }
            } 

            return false;
        }

        #endregion
    }
}